import logging

from fastapi import HTTPException
from fastapi.datastructures import QueryParams
from sqlmodel import select

from openec2.libvirt import LibvirtSingleton
from openec2.config import OpenEC2Config
from openec2.db import DatabaseDep
from openec2.db.instance import Instance
from openec2.utils.array import parse_array_plain


logger = logging.getLogger()

def terminate_instances(
    params: QueryParams,
    config: OpenEC2Config,
    db: DatabaseDep,
):
    conn = LibvirtSingleton.of().connection
    for instance_id in parse_array_plain("InstanceId", params):
        instance = db.exec(select(Instance).where(Instance.id == instance_id)).first()
        if instance is None:
            raise HTTPException(status_code=404, detail="Unknown instance")

        dom = conn.lookupByName(instance_id)
        if dom.isActive():
            dom.shutdown()
        dom.undefine()

        # Delete files
        logger.debug(f"Removing {config.instances.location / instance_id}")
        instance_disk = config.instances.location / instance_id
        instance_disk.unlink()

        db.delete(instance)
        db.commit()

    return "OK"
